#!/usr/bin/python
# Sudoku.py
"""
    Copyright (C) 2010  Peter Hewitt

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

"""
import g,sud,pygame,utils,sys,buttons,load_save,rc
try:
    import gtk
except:
    pass

class Sudoku:

    def __init__(self):
        self.journal=True # set to False if we come in via main()
        self.canvas=None # set to the pygame canvas if we come in via activity.py

    def display(self):
        grey=100; g.screen.fill((grey,grey,grey))
        buttons.draw()
        g.screen.blit(g.help1,g.help1_xy)
        utils.display_number(g.count,(g.xc,g.yc),g.font2,utils.CREAM)
        colour=utils.YELLOW
        if g.helped>5: colour=utils.RED
        utils.display_number(g.helped,(g.xc,g.yc+g.sy(6)),g.font1,colour)
        if g.finished:
            utils.centre_blit(g.screen,g.smiley,(g.sx(4),g.smiley_y))
        self.board.draw()

    def do_button(self,bu):
        if bu=='reset': self.board.reset(); buttons.off('reset')
        if bu=='new': self.board.setup(); buttons.off('reset')

    def do_key(self,key):
        if key in g.SQUARE:
            if not g.spacebar: g.spacebar=True; return
            g.spacebar=False
            self.do_button('new'); return
        g.spacebar=False
        if key in g.CIRCLE:
            if buttons.active('reset'): self.do_button('reset'); return
        if not g.finished:
            if key in g.NUMBERS:
                n=g.NUMBERS[key]; self.board.key(n); return
            if key in g.RIGHT:
                g.green=self.rc.inc_c(g.green); self.board.mouse_set(); return
            if key in g.LEFT:
                g.green=self.rc.dec_c(g.green); self.board.mouse_set(); return
            if key in g.UP:
                g.green=self.rc.dec_r(g.green); self.board.mouse_set(); return
            if key in g.DOWN:
                g.green=self.rc.inc_r(g.green); self.board.mouse_set(); return
        if key==pygame.K_v: g.version_display=not g.version_display; return
        #if key==pygame.K_s: self.board.solve()###
        
    def help_check(self):
        if utils.mouse_on_img(g.help1,g.help1_xy):
            g.glow_sq=None # turn off any glow
            self.display() # no cursor
            utils.centre_blit(g.screen,g.thinking,g.thinking_xy)
            pygame.display.flip()
            self.board.find_move()
            # flush event queue
            while gtk.events_pending(): gtk.main_iteration()
            for event in pygame.event.get(): pass
            return True
        return False

    def buttons_setup(self):
        d=g.d+g.space; x=g.xc; y=g.y0+d; dy=d
        buttons.Button('reset',(x,y),True); y+=dy
        buttons.Button('new',(x,y),True); y+=dy
        g.yc=y+d+d # for count
        if g.started==0:
            buttons.off('reset') # turned on when tile placed by sud.click()

    def flush_queue(self):
        flushing=True
        while flushing:
            flushing=False
            if self.journal:
                while gtk.events_pending(): gtk.main_iteration()
            for event in pygame.event.get(): flushing=True

    def run(self):
        g.init()
        if not self.journal: utils.load()
        self.board=sud.Board()
        self.board.setup(); self.board.mouse_set()
        load_save.retrieve()
        self.board.check()
        self.rc=rc.RC(9,9)
        self.buttons_setup()
        if g.started: buttons.on('reset')
        if self.canvas<>None: self.canvas.grab_focus()
        ctrl=False
        pygame.key.set_repeat(600,120); key_ms=pygame.time.get_ticks()
        going=True
        while going:
            if self.journal:
                # Pump GTK messages.
                while gtk.events_pending(): gtk.main_iteration()

            # Pump PyGame messages.
            for event in pygame.event.get():
                if event.type==pygame.QUIT: # only in standalone version
                    if not self.journal: utils.save()
                    going=False
                elif event.type == pygame.MOUSEMOTION:
                    g.pos=event.pos
                    g.redraw=True
                    if self.canvas<>None: self.canvas.grab_focus()
                    self.board.green_move()
                elif event.type == pygame.MOUSEBUTTONDOWN:
                    g.redraw=True
                    if event.button==1:
                        bu=buttons.check()
                        if bu!='':
                            self.do_button(bu)
                        elif not g.finished:
                            if not self.help_check():
                                self.board.click()
                                if g.finished:
                                    buttons.on('new'); buttons.off('reset')
                    if event.button==3:
                        if buttons.active('reset'): self.do_button('reset')
                    self.flush_queue()
                elif event.type == pygame.KEYDOWN:
                    # throttle keyboard repeat
                    if pygame.time.get_ticks()-key_ms>110:
                        key_ms=pygame.time.get_ticks()
                        if ctrl:
                            if event.key==pygame.K_q:
                                if not self.journal: utils.save()
                                going=False; break
                            else:
                                ctrl=False
                        if event.key in (pygame.K_LCTRL,pygame.K_RCTRL):
                            ctrl=True; break
                        self.do_key(event.key); g.redraw=True
                        self.flush_queue()
                elif event.type == pygame.KEYUP:
                    ctrl=False
            if not going: break
            if g.redraw:
                self.display()
                if g.version_display: utils.version_display()
                g.screen.blit(g.pointer,g.pos)
                pygame.display.flip()
                g.redraw=False
            g.clock.tick(40)

if __name__=="__main__":
    pygame.init()
    pygame.display.set_mode((1024,768),pygame.FULLSCREEN)
    game=Sudoku()
    game.journal=False
    game.run()
    pygame.display.quit()
    pygame.quit()
    sys.exit(0)
